@@ -75,11 +75,13 @@ modules = [
     "issuu",
     "itaku",
     "itchio",
+    "jpgfish",
     "kabeuchi",
     "keenspot",
     "kemonoparty",
     "khinsider",
     "komikcast",
+    "lensdump",
     "lexica",
     "lightroom",
     "lineblog",
@@ -92,6 +94,7 @@ modules = [
     "mangakakalot",
     "manganelo",
     "mangapark",
+    "mangaread",
     "mangasee",
     "mangoxo",
     "misskey",
