@@ -52,8 +52,11 @@ class FanboxExtractor(Extractor):
             url = text.ensure_http_scheme(url)
             body = self.request(url, headers=headers).json()["body"]
             for item in body["items"]:
-                yield self._get_post_data(item["id"])
-
+                try:
+                    yield self._get_post_data(item["id"])
+                except Exception as exc:
+                    self.log.warning("Skipping post %s (%s: %s)",
+                                     item["id"], exc.__class__.__name__, exc)
             url = body["nextUrl"]
 
     def _get_post_data(self, post_id):
@@ -89,6 +92,7 @@ class FanboxExtractor(Extractor):
                     content_body["imageMap"] = {
                         image_id: image_map[image_id]
                         for image_id in images
+                        if image_id in image_map
                     }
 
                 post["content"] = "\n".join(content)
@@ -256,7 +260,6 @@ class FanboxCreatorExtractor(FanboxExtractor):
 
     def posts(self):
         url = "https://api.fanbox.cc/post.listCreator?creatorId={}&limit=10"
-
         return self._pagination(url.format(self.creator_id))
 
 
