@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Copyright 2018-2020 Leonardo Taccari
-# Copyright 2018-2022 Mike Fährmann
+# Copyright 2018-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -40,6 +40,7 @@ class InstagramExtractor(Extractor):
         self._logged_in = True
         self._find_tags = re.compile(r"#\w+").findall
         self._cursor = None
+        self._user = None
 
     def items(self):
         self.login()
@@ -54,12 +55,17 @@ class InstagramExtractor(Extractor):
         previews = self.config("previews", False)
         video_headers = {"User-Agent": "Mozilla/5.0"}
 
+        order = self.config("order-files")
+        reverse = order[0] in ("r", "d") if order else False
+
         for post in self.posts():
 
             if "__typename" in post:
                 post = self._parse_post_graphql(post)
             else:
                 post = self._parse_post_rest(post)
+            if self._user:
+                post["user"] = self._user
             post.update(data)
             files = post.pop("_files")
 
@@ -68,6 +74,8 @@ class InstagramExtractor(Extractor):
 
             if "date" in post:
                 del post["date"]
+            if reverse:
+                files.reverse()
 
             for file in files:
                 file.update(post)
@@ -90,6 +98,11 @@ class InstagramExtractor(Extractor):
     def posts(self):
         return ()
 
+    def finalize(self):
+        if self._cursor:
+            self.log.info("Use '-o cursor=%s' to continue downloading "
+                          "from the current position", self._cursor)
+
     def request(self, url, **kwargs):
         response = Extractor.request(self, url, **kwargs)
 
@@ -104,9 +117,6 @@ class InstagramExtractor(Extractor):
                 page = None
 
             if page:
-                if self._cursor:
-                    self.log.info("Use '-o cursor=%s' to continue downloading "
-                                  "from the current position", self._cursor)
                 raise exception.StopExtraction("HTTP redirect to %s page (%s)",
                                                page, url.partition("?")[0])
 
@@ -114,6 +124,10 @@ class InstagramExtractor(Extractor):
         if www_claim is not None:
             self.www_claim = www_claim
 
+        csrf_token = response.cookies.get("csrftoken")
+        if csrf_token:
+            self.csrf_token = csrf_token
+
         return response
 
     def login(self):
@@ -357,6 +371,22 @@ class InstagramExtractor(Extractor):
         self._cursor = cursor
         return cursor
 
+    def _assign_user(self, user):
+        self._user = user
+
+        for key, old in (
+                ("count_media"     , "edge_owner_to_timeline_media"),
+                ("count_video"     , "edge_felix_video_timeline"),
+                ("count_saved"     , "edge_saved_media"),
+                ("count_mutual"    , "edge_mutual_followed_by"),
+                ("count_follow"    , "edge_follow"),
+                ("count_followed"  , "edge_followed_by"),
+                ("count_collection", "edge_media_collections")):
+            try:
+                user[key] = user.pop(old)["count"]
+            except Exception:
+                user[key] = 0
+
 
 class InstagramUserExtractor(InstagramExtractor):
     """Extractor for an Instagram user profile"""
@@ -731,10 +761,20 @@ class InstagramRestAPI():
         endpoint = "/v1/guides/guide/{}/".format(guide_id)
         return self._pagination_guides(endpoint)
 
-    def highlights_media(self, user_id):
-        chunk_size = 5
+    def highlights_media(self, user_id, chunk_size=5):
         reel_ids = [hl["id"] for hl in self.highlights_tray(user_id)]
 
+        order = self.extractor.config("order-posts")
+        if order:
+            if order in ("desc", "reverse"):
+                reel_ids.reverse()
+            elif order in ("id", "id_asc"):
+                reel_ids.sort(key=lambda r: int(r[10:]))
+            elif order == "id_desc":
+                reel_ids.sort(key=lambda r: int(r[10:]), reverse=True)
+            elif order != "asc":
+                self.extractor.log.warning("Unknown posts order '%s'", order)
+
         for offset in range(0, len(reel_ids), chunk_size):
             yield from self.reels_media(
                 reel_ids[offset : offset+chunk_size])
@@ -774,13 +814,17 @@ class InstagramRestAPI():
         params = {"username": screen_name}
         return self._call(endpoint, params=params)["data"]["user"]
 
+    @memcache(keyarg=1)
     def user_by_id(self, user_id):
         endpoint = "/v1/users/{}/info/".format(user_id)
         return self._call(endpoint)["user"]
 
     def user_id(self, screen_name, check_private=True):
         if screen_name.startswith("id:"):
+            if self.extractor.config("metadata"):
+                self.extractor._user = self.user_by_id(screen_name[3:])
             return screen_name[3:]
+
         user = self.user_by_name(screen_name)
         if user is None:
             raise exception.AuthorizationError(
@@ -790,11 +834,17 @@ class InstagramRestAPI():
             name = user["username"]
             s = "" if name.endswith("s") else "s"
             raise exception.StopExtraction("%s'%s posts are private", name, s)
+        self.extractor._assign_user(user)
         return user["id"]
 
     def user_clips(self, user_id):
         endpoint = "/v1/clips/user/"
-        data = {"target_user_id": user_id, "page_size": "50"}
+        data = {
+            "target_user_id": user_id,
+            "page_size": "50",
+            "max_id": None,
+            "include_feed_video": "true",
+        }
         return self._pagination_post(endpoint, data)
 
     def user_collection(self, collection_id):
@@ -820,19 +870,18 @@ class InstagramRestAPI():
     def _call(self, endpoint, **kwargs):
         extr = self.extractor
 
-        url = "https://i.instagram.com/api" + endpoint
+        url = "https://www.instagram.com/api" + endpoint
         kwargs["headers"] = {
+            "Accept"          : "*/*",
             "X-CSRFToken"     : extr.csrf_token,
             "X-Instagram-AJAX": "1006242110",
             "X-IG-App-ID"     : "936619743392459",
             "X-ASBD-ID"       : "198387",
             "X-IG-WWW-Claim"  : extr.www_claim,
-            "Origin"          : extr.root,
+            "X-Requested-With": "XMLHttpRequest",
+            "Alt-Used"        : "www.instagram.com",
             "Referer"         : extr.root + "/",
         }
-        kwargs["cookies"] = {
-            "csrftoken": extr.csrf_token,
-        }
         return extr.request(url, **kwargs).json()
 
     def _pagination(self, endpoint, params=None, media=False):
@@ -851,7 +900,7 @@ class InstagramRestAPI():
                 yield from data["items"]
 
             if not data.get("more_available"):
-                return
+                return extr._update_cursor(None)
             params["max_id"] = extr._update_cursor(data["next_max_id"])
 
     def _pagination_post(self, endpoint, params):
@@ -866,7 +915,7 @@ class InstagramRestAPI():
 
             info = data["paging_info"]
             if not info.get("more_available"):
-                return
+                return extr._update_cursor(None)
             params["max_id"] = extr._update_cursor(info["max_id"])
 
     def _pagination_sections(self, endpoint, params):
@@ -879,7 +928,7 @@ class InstagramRestAPI():
             yield from info["sections"]
 
             if not info.get("more_available"):
-                return
+                return extr._update_cursor(None)
             params["page"] = info["next_page"]
             params["max_id"] = extr._update_cursor(info["next_max_id"])
 
@@ -894,7 +943,7 @@ class InstagramRestAPI():
                 yield from item["media_items"]
 
             if "next_max_id" not in data:
-                return
+                return extr._update_cursor(None)
             params["max_id"] = extr._update_cursor(data["next_max_id"])
 
 
@@ -982,12 +1031,7 @@ class InstagramGraphqlAPI():
             "X-Requested-With": "XMLHttpRequest",
             "Referer"         : extr.root + "/",
         }
-        cookies = {
-            "csrftoken": extr.csrf_token,
-        }
-        return extr.request(
-            url, params=params, headers=headers, cookies=cookies,
-        ).json()["data"]
+        return extr.request(url, params=params, headers=headers).json()["data"]
 
     def _pagination(self, query_hash, variables,
                     key_data="user", key_edge=None):
@@ -1003,7 +1047,7 @@ class InstagramGraphqlAPI():
 
             info = data["page_info"]
             if not info["has_next_page"]:
-                return
+                return extr._update_cursor(None)
             elif not data["edges"]:
                 s = "" if self.item.endswith("s") else "s"
                 raise exception.StopExtraction(
