@@ -13,6 +13,7 @@ from .. import text, util, exception
 from ..cache import cache
 import itertools
 import json
+import re
 
 BASE_PATTERN = r"(?:https?://)?(?:www\.|mobile\.)?(?:[fv]x)?twitter\.com"
 
@@ -45,7 +46,8 @@ class TwitterExtractor(Extractor):
         if not self.config("transform", True):
             self._transform_user = util.identity
             self._transform_tweet = util.identity
-        self._user = self._user_obj = None
+        self._user = None
+        self._user_obj = None
         self._user_cache = {}
         self._init_sizes()
 
@@ -74,6 +76,10 @@ class TwitterExtractor(Extractor):
         else:
             seen_tweets = None
 
+        if self.twitpic:
+            self._find_twitpic = re.compile(
+                r"https?(://twitpic\.com/(?!photos/)\w+)").findall
+
         for tweet in self.tweets():
 
             if "legacy" in tweet:
@@ -230,12 +236,24 @@ class TwitterExtractor(Extractor):
             files.append({"url": url})
 
     def _extract_twitpic(self, tweet, files):
-        for url in tweet["entities"].get("urls", ()):
+        urls = {}
+
+        # collect URLs from entities
+        for url in tweet["entities"].get("urls") or ():
             url = url["expanded_url"]
             if "//twitpic.com/" not in url or "/photos/" in url:
                 continue
             if url.startswith("http:"):
                 url = "https" + url[4:]
+            urls[url] = None
+
+        # collect URLs from text
+        for url in self._find_twitpic(
+                tweet.get("full_text") or tweet.get("text") or ""):
+            urls["https" + url] = None
+
+        # extract actual URLs
+        for url in urls:
             response = self.request(url, fatal=False)
             if response.status_code >= 400:
                 continue
@@ -769,11 +787,24 @@ class TwitterTweetExtractor(TwitterExtractor):
             "pattern": r"https://pbs\.twimg\.com/media/EaK.+=jpg",
             "count": 4,
         }),
+        # different 'user' and 'author' in quoted Tweet (#3922)
+        ("https://twitter.com/web/status/1644907989109751810", {
+            "keyword": {
+                "author": {"id": 321629993         , "name": "Cakes_Comics"},
+                "user"  : {"id": 718928225360080897, "name": "StobiesGalaxy"},
+            },
+        }),
         # TwitPic embeds (#579)
         ("https://twitter.com/i/web/status/112900228289540096", {
             "options": (("twitpic", True), ("cards", False)),
             "pattern": r"https://\w+.cloudfront.net/photos/large/\d+.jpg",
-            "count": 3,
+            "count": 2,  # 1 duplicate
+        }),
+        # TwitPic URL not in 'urls' (#3792)
+        ("https://twitter.com/shimoigusaP/status/8138669971", {
+            "options": (("twitpic", True),),
+            "pattern": r"https://\w+.cloudfront.net/photos/large/\d+.png",
+            "count": 1,
         }),
         # Twitter card (#1005)
         ("https://twitter.com/billboard/status/1306599586602135555", {
@@ -897,7 +928,8 @@ Your reaction.""",
         for tweet in self.api.tweet_detail(tweet_id):
             if tweet["rest_id"] == tweet_id or \
                     tweet.get("_retweet_id_str") == tweet_id:
-                self._assign_user(tweet["core"]["user_results"]["result"])
+                if self._user_obj is None:
+                    self._assign_user(tweet["core"]["user_results"]["result"])
                 tweets.append(tweet)
 
                 tweet_id = tweet["legacy"].get("quoted_status_id_str")
@@ -1561,9 +1593,9 @@ class TwitterAPI():
 
                 if esw("tweet-"):
                     tweets.append(entry)
-                elif esw("homeConversation-"):
-                    tweets.extend(entry["content"]["items"])
-                elif esw("conversationthread-"):
+                elif esw(("homeConversation-",
+                          "profile-conversation-",
+                          "conversationthread-")):
                     tweets.extend(entry["content"]["items"])
                 elif esw("tombstone-"):
                     item = entry["content"]["itemContent"]
