@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2021-2022 Mike Fährmann
+# Copyright 2021-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,13 +9,12 @@
 """String formatters"""
 
 import os
-import json
+import sys
 import time
 import string
 import _string
 import datetime
 import operator
-import functools
 from . import text, util
 
 NONE = util.NONE
@@ -36,6 +35,8 @@ def parse(format_string, default=NONE, fmt=format):
 
         if kind == "T":
             cls = TemplateFormatter
+        elif kind == "TF":
+            cls = TemplateFStringFormatter
         elif kind == "E":
             cls = ExpressionFormatter
         elif kind == "M":
@@ -199,15 +200,6 @@ class StringFormatter():
             return lambda obj: fmt(conversion(obj))
 
 
-class TemplateFormatter(StringFormatter):
-    """Read format_string from file"""
-
-    def __init__(self, path, default=NONE, fmt=format):
-        with open(util.expand_path(path)) as fp:
-            format_string = fp.read()
-        StringFormatter.__init__(self, format_string, default, fmt)
-
-
 class ExpressionFormatter():
     """Generate text by evaluating a Python expression"""
 
@@ -220,7 +212,7 @@ class ModuleFormatter():
 
     def __init__(self, function_spec, default=NONE, fmt=None):
         module_name, _, function_name = function_spec.partition(":")
-        module = __import__(module_name)
+        module = util.import_file(module_name)
         self.format_map = getattr(module, function_name)
 
 
@@ -228,7 +220,25 @@ class FStringFormatter():
     """Generate text by evaluating an f-string literal"""
 
     def __init__(self, fstring, default=NONE, fmt=None):
-        self.format_map = util.compile_expression("f'''" + fstring + "'''")
+        self.format_map = util.compile_expression('f"""' + fstring + '"""')
+
+
+class TemplateFormatter(StringFormatter):
+    """Read format_string from file"""
+
+    def __init__(self, path, default=NONE, fmt=format):
+        with open(util.expand_path(path)) as fp:
+            format_string = fp.read()
+        StringFormatter.__init__(self, format_string, default, fmt)
+
+
+class TemplateFStringFormatter(FStringFormatter):
+    """Read f-string from file"""
+
+    def __init__(self, path, default=NONE, fmt=format):
+        with open(util.expand_path(path)) as fp:
+            format_string = fp.read()
+        FStringFormatter.__init__(self, format_string, default, fmt)
 
 
 def parse_field_name(field_name):
@@ -246,7 +256,13 @@ def parse_field_name(field_name):
             func = operator.itemgetter
             try:
                 if ":" in key:
-                    key = _slice(key)
+                    if key[0] == "b":
+                        func = _bytesgetter
+                        key = _slice(key[1:])
+                    else:
+                        key = _slice(key)
+                else:
+                    key = key.strip("\"'")
             except TypeError:
                 pass  # key is an integer
 
@@ -265,6 +281,14 @@ def _slice(indices):
     )
 
 
+def _bytesgetter(slice, encoding=sys.getfilesystemencoding()):
+
+    def apply_slice_bytes(obj):
+        return obj.encode(encoding)[slice].decode(encoding, "ignore")
+
+    return apply_slice_bytes
+
+
 def _build_format_func(format_spec, default):
     if format_spec:
         return _FORMAT_SPECIFIERS.get(
@@ -284,11 +308,20 @@ def _parse_optional(format_spec, default):
 
 def _parse_slice(format_spec, default):
     indices, _, format_spec = format_spec.partition("]")
-    slice = _slice(indices[1:])
     fmt = _build_format_func(format_spec, default)
 
-    def apply_slice(obj):
-        return fmt(obj[slice])
+    if indices[1] == "b":
+        slice_bytes = _bytesgetter(_slice(indices[2:]))
+
+        def apply_slice(obj):
+            return fmt(slice_bytes(obj))
+
+    else:
+        slice = _slice(indices[1:])
+
+        def apply_slice(obj):
+            return fmt(obj[slice])
+
     return apply_slice
 
 
@@ -399,7 +432,7 @@ _CONVERSIONS = {
     "u": str.upper,
     "c": str.capitalize,
     "C": string.capwords,
-    "j": functools.partial(json.dumps, default=str),
+    "j": util.json_dumps,
     "t": str.strip,
     "T": util.datetime_to_timestamp_string,
     "d": text.parse_timestamp,
