@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-# Copyright 2021-2022 Mike Fährmann
+# Copyright 2021-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -23,6 +23,7 @@ class TestFormatter(unittest.TestCase):
     kwdict = {
         "a": "hElLo wOrLd",
         "b": "äöü",
+        "j": "げんそうきょう",
         "d": {"a": "foo", "b": 0, "c": None},
         "l": ["a", "b", "c"],
         "n": None,
@@ -58,6 +59,7 @@ class TestFormatter(unittest.TestCase):
         self._run_test("{dt!T}", "1262304000")
         self._run_test("{l!j}", '["a", "b", "c"]')
         self._run_test("{dt!j}", '"2010-01-01 00:00:00"')
+        self._run_test("{a!g}", "hello-world")
 
         with self.assertRaises(KeyError):
             self._run_test("{a!q}", "hello world")
@@ -127,7 +129,12 @@ class TestFormatter(unittest.TestCase):
         self._run_test("{l[0]}" , "a")
         self._run_test("{a[6]}" , "w")
 
-    def test_slicing(self):
+    def test_dict_access(self):
+        self._run_test("{d[a]}"  , "foo")
+        self._run_test("{d['a']}", "foo")
+        self._run_test('{d["a"]}', "foo")
+
+    def test_slice_str(self):
         v = self.kwdict["a"]
         self._run_test("{a[1:10]}"  , v[1:10])
         self._run_test("{a[-10:-1]}", v[-10:-1])
@@ -159,6 +166,26 @@ class TestFormatter(unittest.TestCase):
         self._run_test("{a:[:50:2]}", v[:50:2])
         self._run_test("{a:[::]}"   , v)
 
+    def test_slice_bytes(self):
+        v = self.kwdict["j"]
+        self._run_test("{j[b1:10]}"  , v[1:3])
+        self._run_test("{j[b-10:-1]}", v[-3:-1])
+        self._run_test("{j[b5:]}"    , v[2:])
+        self._run_test("{j[b50:]}"   , v[50:])
+        self._run_test("{j[b:5]}"    , v[:1])
+        self._run_test("{j[b:50]}"   , v[:50])
+        self._run_test("{j[b:]}"     , v)
+        self._run_test("{j[b::]}"    , v)
+
+        self._run_test("{j:[b1:10]}"  , v[1:3])
+        self._run_test("{j:[b-10:-1]}", v[-3:-1])
+        self._run_test("{j:[b5:]}"    , v[2:])
+        self._run_test("{j:[b50:]}"   , v[50:])
+        self._run_test("{j:[b:5]}"    , v[:1])
+        self._run_test("{j:[b:50]}"   , v[:50])
+        self._run_test("{j:[b:]}"     , v)
+        self._run_test("{j:[b::]}"    , v)
+
     def test_maxlen(self):
         v = self.kwdict["a"]
         self._run_test("{a:L5/foo/}" , "foo")
@@ -177,7 +204,7 @@ class TestFormatter(unittest.TestCase):
         self._run_test("{l:J - />20}", "           a - b - c")
 
         self._run_test("{a:J/}"      , self.kwdict["a"])
-        self._run_test("{a:J, /}"    , ", ".join(self.kwdict["a"]))
+        self._run_test("{a:J, /}"    , self.kwdict["a"])
 
     def test_replace(self):
         self._run_test("{a:Rh/C/}"  , "CElLo wOrLd")
@@ -218,6 +245,21 @@ class TestFormatter(unittest.TestCase):
             time.timezone = orig_timezone
             time.altzone = orig_altzone
 
+    def test_sort(self):
+        self._run_test("{l:S}" , "['a', 'b', 'c']")
+        self._run_test("{l:Sa}", "['a', 'b', 'c']")
+        self._run_test("{l:Sd}", "['c', 'b', 'a']")
+        self._run_test("{l:Sr}", "['c', 'b', 'a']")
+
+        self._run_test(
+            "{a:S}", "[' ', 'E', 'L', 'L', 'O', 'd', 'h', 'l', 'o', 'r', 'w']")
+        self._run_test(
+            "{a:S-asc}",  # starts with 'S', contains 'a'
+            "[' ', 'E', 'L', 'L', 'O', 'd', 'h', 'l', 'o', 'r', 'w']")
+        self._run_test(
+            "{a:Sort-reverse}",  # starts with 'S', contains 'r'
+            "['w', 'r', 'o', 'l', 'h', 'd', 'O', 'L', 'L', 'E', ' ']")
+
     def test_chain_special(self):
         # multiple replacements
         self._run_test("{a:Rh/C/RE/e/RL/l/}", "Cello wOrld")
@@ -236,6 +278,9 @@ class TestFormatter(unittest.TestCase):
         # parse and format datetime
         self._run_test("{ds:D%Y-%m-%dT%H:%M:%S%z/%Y%m%d}", "20100101")
 
+        # sort and join
+        self._run_test("{a:S/J}", " ELLOdhlorw")
+
     def test_separator(self):
         orig_separator = formatter._SEPARATOR
         try:
@@ -329,6 +374,27 @@ class TestFormatter(unittest.TestCase):
         self._run_test("\fF foo-'\"{a.upper()}\"'-bar",
                        """foo-'"{}"'-bar""".format(self.kwdict["a"].upper()))
 
+    @unittest.skipIf(sys.hexversion < 0x3060000, "no fstring support")
+    def test_template_fstring(self):
+        with tempfile.TemporaryDirectory() as tmpdirname:
+            path1 = os.path.join(tmpdirname, "tpl1")
+            path2 = os.path.join(tmpdirname, "tpl2")
+
+            with open(path1, "w") as fp:
+                fp.write("{a}")
+            fmt1 = formatter.parse("\fTF " + path1)
+
+            with open(path2, "w") as fp:
+                fp.write("foo-'\"{a.upper()}\"'-bar")
+            fmt2 = formatter.parse("\fTF " + path2)
+
+        self.assertEqual(fmt1.format_map(self.kwdict), self.kwdict["a"])
+        self.assertEqual(fmt2.format_map(self.kwdict),
+                         """foo-'"{}"'-bar""".format(self.kwdict["a"].upper()))
+
+        with self.assertRaises(OSError):
+            formatter.parse("\fTF /")
+
     def test_module(self):
         with tempfile.TemporaryDirectory() as tmpdirname:
             path = os.path.join(tmpdirname, "testmod.py")
@@ -355,7 +421,7 @@ def noarg():
             try:
                 fmt1 = formatter.parse("\fM testmod:gentext")
                 fmt2 = formatter.parse("\fM testmod:lengths")
-                fmt3 = formatter.parse("\fM testmod:noarg")
+                fmt0 = formatter.parse("\fM testmod:noarg")
 
                 with self.assertRaises(AttributeError):
                     formatter.parse("\fM testmod:missing")
@@ -364,11 +430,17 @@ def noarg():
             finally:
                 sys.path.pop(0)
 
+            fmt3 = formatter.parse("\fM " + path + ":gentext")
+            fmt4 = formatter.parse("\fM " + path + ":lengths")
+
         self.assertEqual(fmt1.format_map(self.kwdict), "'Title' by Name")
-        self.assertEqual(fmt2.format_map(self.kwdict), "89")
+        self.assertEqual(fmt2.format_map(self.kwdict), "96")
+
+        self.assertEqual(fmt3.format_map(self.kwdict), "'Title' by Name")
+        self.assertEqual(fmt4.format_map(self.kwdict), "96")
 
         with self.assertRaises(TypeError):
-            self.assertEqual(fmt3.format_map(self.kwdict), "")
+            self.assertEqual(fmt0.format_map(self.kwdict), "")
 
     def _run_test(self, format_string, result, default=None, fmt=format):
         fmt = formatter.parse(format_string, default, fmt)
